@@ -1,6 +1,6 @@
 """
-The `Code` object is responsible for parsing a document, received from the `Line` class.
-It's usually tokenized, using a Pygments lexer.
+The `Code` object is responsible for parsing a document, received from the
+`Line` class.  It's usually tokenized, using a Pygments lexer.
 """
 from __future__ import unicode_literals
 from pygments.token import Token
@@ -8,7 +8,7 @@ from pygments.token import Token
 __all__ = (
     'Code',
     'CodeBase',
-    'Completion'
+    'Completion',
     'ValidationError',
 )
 
@@ -74,7 +74,7 @@ class CodeBase(object):
         def get_suffix(completion):
             return completion.text[-completion.start_position:]
 
-        return _commonprefix([ get_suffix(c) for c in completions ])
+        return _commonprefix([get_suffix(c) for c in completions])
 
     def get_completions(self):
         """ Yield `Completion` instances. """
@@ -89,7 +89,6 @@ class CodeBase(object):
         pass
 
 
-
 class Code(CodeBase):
     """
     Representation of a code document.
@@ -109,9 +108,9 @@ class Code(CodeBase):
         """ Return lexer instance. """
         if self.lexer:
             return self.lexer(
-                    stripnl=False,
-                    stripall=False,
-                    ensurenl=False)
+                stripnl=False,
+                stripall=False,
+                ensurenl=False)
         else:
             return None
 
