@@ -22,7 +22,7 @@ from prompt_toolkit.key_bindings.vi import vi_bindings
 from prompt_toolkit.keys import Keys
 from prompt_toolkit.line import Line
 from prompt_toolkit.prompt import Prompt, TokenList, BracketsMismatchProcessor, PopupCompletionMenu, HorizontalCompletionMenu
-from prompt_toolkit.renderer import Char
+from prompt_toolkit.renderer import Char, Screen, Point
 from prompt_toolkit.selection import SelectionType
 
 import jedi
@@ -39,7 +39,7 @@ __all__ = (
 
 class AutoCompletionStyle:
     #: tab/double-tab completion
-    # TRADITIONAL = 'traditional' # TODO: not implemented yet.
+    # TRADITIONAL = 'traditional'  # TODO: not implemented yet.
 
     #: Pop-up
     POPUP_MENU = 'popup-menu'
@@ -47,7 +47,7 @@ class AutoCompletionStyle:
     #: Horizontal list
     HORIZONTAL_MENU = 'horizontal-menu'
 
-    #:Pop-up menu that also displays the references to the Python modules.
+    #: Pop-up menu that also displays the references to the Python modules.
     EXTENDED_POPUP_MENU = 'extended-popup-menu'
 
     #: No visualisation
@@ -120,6 +120,7 @@ class PythonStyle(Style):
         Token.Leftmargin.Tilde:   '#888888',
     }
 
+
 def _has_unclosed_brackets(text):
     """
     Starting at the end of the string. If we find an opening bracket
@@ -128,7 +129,7 @@ def _has_unclosed_brackets(text):
     stack = []
 
     # Ignore braces inside strings
-    text = re.sub(r'''('[^']*'|"[^"]*")''', '', text) # XXX: handle escaped quotes.!
+    text = re.sub(r'''('[^']*'|"[^"]*")''', '', text)  # XXX: handle escaped quotes.!
 
     for c in reversed(text):
         if c in '])}':
@@ -137,8 +138,8 @@ def _has_unclosed_brackets(text):
         elif c in '[({':
             if stack:
                 if ((c == '[' and stack[-1] == ']') or
-                    (c == '{' and stack[-1] == '}') or
-                    (c == '(' and stack[-1] == ')')):
+                        (c == '{' and stack[-1] == '}') or
+                        (c == '(' and stack[-1] == ')')):
                     stack.pop()
             else:
                 # Opening bracket for which we didn't had a closing one.
@@ -173,8 +174,10 @@ def python_bindings(registry, cli_ref):
     @handle(Keys.Tab, in_mode=InputMode.INSERT)
     @handle(Keys.Tab, in_mode=InputMode.COMPLETE)
     def _(event):
-        # When the 'tab' key is pressed with only whitespace character before the
-        # cursor, do autocompletion. Otherwise, insert indentation.
+        """
+        When the 'tab' key is pressed with only whitespace character before the
+        cursor, do autocompletion. Otherwise, insert indentation.
+        """
         current_char = line.document.current_line_before_cursor
         if not current_char or current_char.isspace():
             line.insert_text('    ')
@@ -186,44 +189,15 @@ def python_bindings(registry, cli_ref):
     @handle(Keys.BackTab, in_mode=InputMode.INSERT)
     @handle(Keys.BackTab, in_mode=InputMode.COMPLETE)
     def _(event):
+        """
+        Shift+Tab: go to previous completion.
+        """
         line.complete_previous()
 
         if event.input_processor.input_mode != InputMode.COMPLETE:
             event.input_processor.push_input_mode(InputMode.COMPLETE)
             line.complete_previous()
 
-    @handle(Keys.ControlJ, in_mode=InputMode.INSERT)
-    @handle(Keys.ControlM, in_mode=InputMode.INSERT)
-    def _(event):
-        _auto_enable_multiline()
-
-        if line.is_multiline:
-            line.newline()
-        else:
-            line.return_input()
-
-    def _auto_enable_multiline():
-        """
-        (Temporarily) enable multiline when pressing enter.
-        When:
-        - We press [enter] after a color or backslash (line continuation).
-        - After unclosed brackets.
-        """
-        def is_empty_or_space(s):
-            return s == '' or s.isspace()
-        cursor_at_the_end = line.document.is_cursor_at_the_end
-
-        # If we just typed a colon, or still have open brackets, always insert a real newline.
-        if cursor_at_the_end and (line.document.text_before_cursor.rstrip()[-1:] == ':' or
-                                  _has_unclosed_brackets(line.document.text_before_cursor) or
-                                  line.text.startswith('@')):
-            line.is_multiline = True
-
-        # If the character before the cursor is a backslash (line continuation
-        # char), insert a new line.
-        elif cursor_at_the_end and (line.document.text_before_cursor[-1:] == '\\'):
-            line.is_multiline = True
-
 
 class PythonLine(Line):
     """
@@ -246,7 +220,9 @@ class PythonLine(Line):
         self.signatures = []
 
     def text_changed(self):
-        self.is_multiline = '\n' in self.text
+        # When there is '\n' in the input, or in case of paste mode, always
+        # make sure that we enable multiline.
+        self.is_multiline = '\n' in self.text or self.paste_mode
 
     def newline(self):
         r"""
@@ -273,15 +249,43 @@ class PythonLine(Line):
                 for x in range(4):
                     insert_text(' ')
 
+    def auto_enter(self):
+        self._auto_enable_multiline()
+        super(PythonLine, self).auto_enter()
+
+    def _auto_enable_multiline(self):
+        """
+        (Temporarily) enable multiline when pressing enter.
+        When:
+        - We press [enter] after a color or backslash (line continuation).
+        - After unclosed brackets.
+        """
+        def is_empty_or_space(s):
+            return s == '' or s.isspace()
+        cursor_at_the_end = self.document.is_cursor_at_the_end
+
+        # If we just typed a colon, or still have open brackets, always insert a real newline.
+        if cursor_at_the_end and (self.document.text_before_cursor.rstrip()[-1:] == ':' or
+                                  _has_unclosed_brackets(self.document.text_before_cursor) or
+                                  self.text.startswith('@')):
+            self.is_multiline = True
+
+        # If the character before the cursor is a backslash (line continuation
+        # char), insert a new line.
+        elif cursor_at_the_end and (self.document.text_before_cursor[-1:] == '\\'):
+            self.is_multiline = True
+
 
 class PythonPrompt(Prompt):
-    input_processors = [ BracketsMismatchProcessor() ]
+    input_processors = [BracketsMismatchProcessor()]
 
     min_height = 7
 
-    @property
-    def tokens_before_input(self):
-        return [(Token.Prompt, 'In [%s]: ' % self.commandline.current_statement_index)]
+    def __init__(self, commandline_ref):
+        super(PythonPrompt, self).__init__(commandline_ref)
+
+        #: Vertical scrolling position of the main content.
+        self.vertical_scroll = 0
 
     @property
     def completion_menu(self):
@@ -311,8 +315,8 @@ class PythonPrompt(Prompt):
             screen.write_highlighted(list(self._get_error_tokens()))
 
         elif self.commandline.autocompletion_style == AutoCompletionStyle.HORIZONTAL_MENU and \
-                        self.line.complete_state and \
-                        self.commandline.input_processor.input_mode == InputMode.COMPLETE:
+                self.line.complete_state and \
+                self.commandline.input_processor.input_mode == InputMode.COMPLETE:
             HorizontalCompletionMenu().write(screen, None, self.line.complete_state)
         else:
             screen.write_highlighted(list(self._get_signature_tokens()))
@@ -323,7 +327,7 @@ class PythonPrompt(Prompt):
         Signature = Token.Signature
 
         if self.line.signatures:
-            sig = self.line.signatures[0] # Always take the first one.
+            sig = self.line.signatures[0]  # Always take the first one.
 
             append((Token, '           '))
             append((Signature, sig.full_name))
@@ -336,67 +340,101 @@ class PythonPrompt(Prompt):
                     append((Signature, str(p.name)))
                 append((Signature.Operator, ', '))
 
-            result.pop() # Pop last comma
+            result.pop()  # Pop last comma
             append((Signature.Operator, ')'))
         return result
 
     def _get_error_tokens(self):
         if self.line.validation_error:
             text = '%s (line=%s column=%s)' % (
-                    self.line.validation_error.message,
-                    self.line.validation_error.line + 1,
-                    self.line.validation_error.column + 1)
+                self.line.validation_error.message,
+                self.line.validation_error.line + 1,
+                self.line.validation_error.column + 1)
             return [(Token.ValidationError, text)]
         else:
             return []
 
-    def create_left_input_margin(self, screen, row, is_new_line):
-        prompt_width = len(TokenList(self.tokens_before_input))
+    @property
+    def prompt_text(self):
+        return 'In [%s]: ' % self.commandline.current_statement_index
 
+    def write_prompt(self, screen):
+        screen.write_highlighted_at_pos(0, 0, [(Token.Prompt, self.prompt_text)])
+
+    def create_left_input_margin(self, screen, row, is_new_line):
         if is_new_line:
             text = '%i. ' % row
         else:
             text = ''
 
-        text = ' ' * (prompt_width - len(text)) + text
-
         screen.write_highlighted([
-            (Token.Prompt.LineNumber, text)
+            (Token.Prompt.LineNumber, ' ' * (len(self.prompt_text) - len(text))),
+            (Token.Prompt.LineNumber, text),
         ])
 
-    def _get_bottom_position(self, screen, last_screen_height):
-        # Draw the menu at the bottom position.
-        return max(self.min_height - 2, screen.current_height, last_screen_height - 2) - 1
+    def write_input_scrolled(self, screen, accept_or_abort, last_screen_height):
+        # Write to a temp screen first.
+        temp_screen = Screen(screen.size)
+        super(PythonPrompt, self).write_input(temp_screen, highlight=not accept_or_abort)
 
-    def _print_tildes(self, screen):
-        """
-        Print tildes in the left margin between the last input line and the
-        toolbars.
-        """
-        last_char_y, last_char_x = screen._cursor_mappings[len(self.line.document.text)]
+        # Determine the maximum height.
+        max_height = screen.size.rows - 2
 
-        # Fill space with tildes
-        for y in range(last_char_y + 1, screen.current_height - 2):
-            screen.write_at_pos(y, 1, Char('~', Token.Leftmargin.Tilde))
+        if True:
+            # Scroll back if we scrolled to much and there's still space at the top.
+            if self.vertical_scroll > temp_screen.current_height - max_height:
+                self.vertical_scroll = max(0, temp_screen.current_height - max_height)
+
+            # Scroll up if cursor is before visible part.
+            if self.vertical_scroll > temp_screen.cursor_position.y:
+                self.vertical_scroll = temp_screen.cursor_position.y
+
+            # Scroll down if cursor is after visible part.
+            if self.vertical_scroll <= temp_screen.cursor_position.y - max_height:
+                self.vertical_scroll = (temp_screen.cursor_position.y + 1) - max_height
+
+            # Scroll down if we need space for the menu.
+            if self.need_to_show_completion_menu():
+                menu_size = min(5, len(self.line.complete_state.current_completions)) - 1
+                if temp_screen.cursor_position.y - self.vertical_scroll >= max_height - menu_size:
+                    self.vertical_scroll = (temp_screen.cursor_position.y + 1) - (max_height - menu_size)
+
+        # Now copy the region we need to the real screen.
+        y = 0
+        for y in range(0, min(max_height, temp_screen.current_height - self.vertical_scroll)):
+            for x in range(0, temp_screen.size.columns):
+                screen._buffer[y][x] = temp_screen._buffer[y + self.vertical_scroll][x]
+
+        screen.cursor_position = Point(y=temp_screen.cursor_position.y - self.vertical_scroll,
+                                       x=temp_screen.cursor_position.x)
+
+        # Fill up with tildes.
+        if not accept_or_abort:
+            y += 1
+            while y < max([self.min_height - 2, last_screen_height - 2]) and y < max_height:
+                screen.write_at_pos(y, 1, Char('~', Token.Leftmargin.Tilde))
+                y += 1
+
+        return_y = y
+
+        # Show completion menu.
+        if not accept_or_abort and self.need_to_show_completion_menu():
+            y, x = temp_screen._cursor_mappings[self.line.complete_state.original_document.cursor_position]
+            self.completion_menu.write(screen, (y - self.vertical_scroll, x), self.line.complete_state)
+
+        return return_y
 
     def write_to_screen(self, screen, last_screen_height, accept=False, abort=False):
-        self.write_before_input(screen)
-        self.write_input(screen, highlight=not (accept or abort))
+        y = self.write_input_scrolled(screen, (accept or abort), last_screen_height)
+        self.write_prompt(screen)
 
         if not (accept or abort):
-            y = self._get_bottom_position(screen, last_screen_height)
-            self.write_menus(screen) # XXX: menu should be able to cover the second toolbar.
-            y2 = self._get_bottom_position(screen, last_screen_height)
-            y = max(y, y2 - 1)
-
-            screen._y, screen._x = y + 1, 0
+            screen._y, screen._x = y, 0
             self.write_second_toolbar(screen)
 
-            screen._y, screen._x = y + 2, 0
+            screen._y, screen._x = y + 1, 0
             self.write_toolbar(screen)
 
-            self._print_tildes(screen)
-
     def write_toolbar(self, screen):
         TB = Token.Toolbar
         mode = self.commandline.input_processor.input_mode
@@ -434,10 +472,6 @@ class PythonPrompt(Prompt):
                     append((TB.Mode, '(VISUAL)'))
                     append((TB, ' '))
 
-#            if self.commandline._input_processor.is_recording_macro:
-#                append((TB.Mode, 'recording'))
-#                append((TB, ' '))
-
         else:
             append((TB.Mode, '(emacs)'))
             append((TB, ' '))
@@ -469,7 +503,7 @@ class PythonPrompt(Prompt):
             version = sys.version_info
             append((TB, ' - '))
             append((TB.PythonVersion, '%s %i.%i.%i' % (platform.python_implementation(),
-                                version.major, version.minor, version.micro)))
+                   version.major, version.minor, version.micro)))
 
         # Adjust toolbar width.
         if len(result) > screen.size.columns:
@@ -489,9 +523,9 @@ class ExtendedPopupCompletionMenu(PopupCompletionMenu):
     """
     def get_menu_width(self, complete_state):
         return [
-                super(ExtendedPopupCompletionMenu, self).get_menu_width(complete_state),
-                max(len(c.jedi_completion.type) for c in complete_state.current_completions)
-                ]
+            super(ExtendedPopupCompletionMenu, self).get_menu_width(complete_state),
+            max(len(c.jedi_completion.type) for c in complete_state.current_completions)
+        ]
 
     def get_menu_item_tokens(self, completion, is_current_completion, menu_width):
         if is_current_completion:
@@ -500,7 +534,8 @@ class ExtendedPopupCompletionMenu(PopupCompletionMenu):
             token = Token.CompletionMenu.JediDescription
 
         return super(ExtendedPopupCompletionMenu, self).get_menu_item_tokens(completion, is_current_completion, menu_width[0]) + [
-                (token, ' %%-%is ' % menu_width[1] % completion.jedi_completion.type or 'none') ]
+            (token, ' %%-%is ' % menu_width[1] % completion.jedi_completion.type or 'none')
+        ]
 
 
 class PythonCompletion(Completion):
@@ -532,11 +567,12 @@ class PythonCode(Code):
 
     def _get_jedi_script(self):
         try:
-            return jedi.Interpreter(self.text,
-                    column=self.document.cursor_position_col,
-                    line=self.document.cursor_position_row + 1,
-                    path='input-text',
-                    namespaces=[ self._locals, self._globals ])
+            return jedi.Interpreter(
+                self.text,
+                column=self.document.cursor_position_col,
+                line=self.document.cursor_position_row + 1,
+                path='input-text',
+                namespaces=[self._locals, self._globals])
 
         except jedi.common.MultiLevelStopIteration:
             # This happens when the document is just a backslash.
@@ -560,7 +596,7 @@ class PythonCommandLineInterface(CommandLineInterface):
     prompt_factory = PythonPrompt
 
     def __init__(self, globals=None, locals=None, vi_mode=False, stdin=None, stdout=None, history_filename=None,
-                    style=PythonStyle, autocompletion_style=AutoCompletionStyle.POPUP_MENU):
+                 style=PythonStyle, autocompletion_style=AutoCompletionStyle.POPUP_MENU):
 
         self.globals = globals or {}
         self.locals = locals or globals
