@@ -129,7 +129,7 @@ class Document(object):
 
         return row, col
 
-    def translate_row_col_to_index(self, row, col):  # TODO: unit test
+    def translate_row_col_to_index(self, row, col):
         """
         Given a (row, col) tuple, return the corresponding index.
         (Row and col params are 0-based.)
@@ -303,7 +303,7 @@ class Document(object):
         except StopIteration:
             pass
 
-    def find_next_matching_line(self, match_func):  # TODO: unittest.
+    def find_next_matching_line(self, match_func):
         """
         Look downwards for empty lines.
         Return the line index, relative to the current line.
@@ -312,7 +312,7 @@ class Document(object):
             if match_func(line):
                 return 1 + index
 
-    def find_previous_matching_line(self, match_func):  # TODO: unittest.
+    def find_previous_matching_line(self, match_func):
         """
         Look upwards for empty lines.
         Return the line index, relative to the current line.
